/* MQTT_Bench
 *
 *******************************************************************************
 * Project:		MQTT_Bench
 *
 * Author:		Walter Trojan
 *
 * Description: MQTT benchmark, how many MQTT messages per second
 *
 * IDE:			Eclipse
 *
 * Finished:	Aug. 2016
 *
 *
 *******************************************************************************/


#include <ets_sys.h>
#include <osapi.h>
#include <os_type.h>
#include <user_interface.h>
#include <espconn.h>
#include "mqtt.h"
#include "debug.h"
#include "mem.h"
#include "uart.h"
#include "uart_register.h"
#include "config.h"


/*******************************************************************************
 * Global variables
 ******************************************************************************/
MQTT_Client mqttClient;
static ETSTimer Wloop;


bool     Gopub;
uint32_t Systim = 0;
bool	 RdyInit;
bool     RdyCon;
bool	 RdyIP;
bool     RdyMQ;
bool	 RdyPub;
bool     RdySub = false;
char     TopBuf[20];
char     DatBuf[20];
uint32_t Benchcount;
char     msg[64];
uint32_t TimeSta;
uint32_t TimeEnd;
uint32_t TimeGes;
uint32_t Time1;
uint32_t Time2;

/*******************************************************************************
 * WiFi_Connect: Connect station to router
 ******************************************************************************/
void ICACHE_FLASH_ATTR WIFI_Connect(uint8_t* ssid, uint8_t* pass)
{
	struct station_config stationConf;
	bool retval;								//

	INFO("WIFI_INIT\r\n");
	wifi_set_opmode(STATION_MODE);
	wifi_station_disconnect();					//
	wifi_station_dhcpc_stop();					//
	os_memset(&stationConf, 0, sizeof(struct station_config));

	os_sprintf(stationConf.ssid, "%s", ssid);
	//INFO("...SSID=%s\n\r",ssid);				//
	os_sprintf(stationConf.password, "%s", pass);
	//INFO("...PASS=%s\n\r",pass);
	stationConf.bssid_set = 0; 					//need not check MAC address of AP

	retval = wifi_station_set_config(&stationConf);
	if(retval)									//
		INFO("wifi_station_set = OK\n\r");

	retval = wifi_station_connect();
	wifi_station_dhcpc_start();					//
	if(retval)									//
		INFO("wifi_station_connect = OK\n\r");
	else
		INFO("wifi_station_connect = NNNOK\n\r");
}



/*******************************************************************************
 * MQTT callbacks
 ******************************************************************************/
void mqtt_connected_cb(uint32_t *args)
{

	os_delay_us(65000);
	INFO(" ==> MQTT: Connected\r\n");
	MQTT_Client* client = (MQTT_Client*)args;


	TimeSta = system_get_time();
	Time1   = TimeSta;
	Benchcount = 1;
	os_sprintf(msg, "\r\nStart \r\n%4u  ESP01 Bench 123456789012345678901234567890  %8u uS", Benchcount, TimeSta);
	MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);


}

void mqtt_disconnected_cb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	RdyMQ = false;
	INFO(" ==> MQTT: Disconnected\r\n");
}

void mqtt_published_cb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	INFO("MQTT: Published\r\n");

	Benchcount = Benchcount + 1;
	if(Benchcount < 101){
		Time2 = system_get_time();
		os_sprintf(msg, "%4u  ESP01 Bench 123456789012345678901234567890  %8u uS", Benchcount, Time2-Time1);
		MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);
		Time1 = Time2;
	}

	if(Benchcount == 101){
		TimeEnd = system_get_time();
		TimeGes = TimeEnd - TimeSta;
		os_sprintf(msg, "==> Zeit:%8u uS ", TimeGes);
		MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);
	}

}

void mqtt_data_cb(uint32_t *args, const char* topic, uint32_t topic_len, const char *data, uint32_t data_len)
{
	char *topicBuf = (char*)os_zalloc(topic_len+1),
			*dataBuf = (char*)os_zalloc(data_len+1);

	MQTT_Client* client = (MQTT_Client*)args;

	os_memcpy(topicBuf, topic, topic_len);
	topicBuf[topic_len] = 0;

	os_memcpy(dataBuf, data, data_len);
	dataBuf[data_len] = 0;

	INFO("/r/n Receive topic: %s, data: %s \r\n", topicBuf, dataBuf);

	RdySub = true;
	strcpy(TopBuf,topicBuf);
	strcpy(DatBuf,dataBuf);

	os_free(topicBuf);
	os_free(dataBuf);
}


/*******************************************************************************
 * init_done_cb: Called after init is ready
 ******************************************************************************/

static void ICACHE_FLASH_ATTR init_done_cb(void)
{
	os_delay_us(65000);
	INFO("\r\n ==> Init done.\r\n");
	WIFI_Connect(config.sta_ssid, config.sta_pwd);
}


/*******************************************************************************
 * eventHandler: Called after some events
 ******************************************************************************/
static void ICACHE_FLASH_ATTR eventHandler(System_Event_t *event) {
	switch(event->event) {
		case EVENT_STAMODE_CONNECTED:
		{
			INFO(" ==> Event: EVENT_STAMODE_CONNECTED\r\n");
			break;
		}
		case EVENT_STAMODE_DISCONNECTED:
		{
			INFO(" ==> Event: EVENT_STAMODE_DISCONNECTED\r\n");
			break;
		}
		case EVENT_STAMODE_AUTHMODE_CHANGE:
		{
			INFO(" Event: EVENT_STAMODE_AUTHMODE_CHANGE\r\n");
			break;
		}
		case EVENT_STAMODE_GOT_IP:
		{
			INFO(" ==> Event: EVENT_STAMODE_GOT_IP\r\n");
			os_delay_us(65000);
			MQTT_Connect(&mqttClient);
			break;
		}
		case EVENT_SOFTAPMODE_STACONNECTED:
		{
			INFO(" Event: EVENT_SOFTAPMODE_STACONNECTED\r\n");
			break;
		}
		case EVENT_SOFTAPMODE_STADISCONNECTED:
		{
			INFO(" Event: EVENT_SOFTAPMODE_STADISCONNECTED\r\n");
			break;
		}
		default:
		{
			INFO(" Unexpected event: %d\r\n", event->event);
			break;
		}
	}
}





/********************************************************************************
 * sysclk: Will be called cyclic by RTOS, depending on timer Systic (every 10 ms)
 ******************************************************************************/

LOCAL void ICACHE_FLASH_ATTR sysclk(void *arg)
{
	Systim++;
}



/*******************************************************************************
 * workloop: Will be by RTOS, depending on timer Wloop, calling itself, if ready
 ******************************************************************************/

static void ICACHE_FLASH_ATTR workloop(void *arg)
{
	os_timer_disarm(&Wloop);


	os_timer_setfn(&Wloop, (os_timer_func_t *)workloop, NULL);
	os_timer_arm(&Wloop, 10, 0);
}


/*******************************************************************************
 * user_init: Performs once for initialization
 ******************************************************************************/
void user_init(void)
{
	uint8_t cpuspeed;

	uart_init(BIT_RATE_115200, BIT_RATE_115200);
	os_delay_us(1000000);

	cpuspeed = system_get_cpu_freq();
	INFO("\r\n\n ESP8266 platform started!\r\n");
	INFO("\r\n SDK version:%s\n", system_get_sdk_version());
	INFO("\r\n CPU-Freq: %d\r\n",cpuspeed);

	wifi_station_set_auto_connect(0);				// no auto connection
	system_init_done_cb(init_done_cb);				// register init_done_cb
	wifi_set_event_handler_cb(eventHandler);		// register event handler

	config_load();

	RdyInit = false;
	RdyCon  = false;
	RdyIP   = false;
	RdyMQ   = false;
	RdyPub  = false;


	MQTT_InitConnection(&mqttClient, config.mqtt_host, config.mqtt_port, config.security);
	MQTT_InitClient(&mqttClient, config.device_id, config.mqtt_user, config.mqtt_pass, config.mqtt_keepalive, 1);
	MQTT_InitLWT(&mqttClient, "/lwt", "offline", 0, 0);
	MQTT_OnConnected(&mqttClient, mqtt_connected_cb);
	MQTT_OnDisconnected(&mqttClient, mqtt_disconnected_cb);
	MQTT_OnPublished(&mqttClient, mqtt_published_cb);
	MQTT_OnData(&mqttClient, mqtt_data_cb);


/*
	os_timer_disarm(&Wloop);
	os_timer_setfn(&Wloop, (os_timer_func_t *)workloop, NULL);
	os_timer_arm(&Wloop, 10, 0);

	os_timer_disarm(&Systic);
	os_timer_setfn(&Systic, (os_timer_func_t *)sysclk, NULL);
	os_timer_arm(&Systic, 10, 1);
*/
	INFO("\r\nSystem started ...\r\n");
}

//void user_rf_pre_init(void) {}
