/* MQTTAct1
 *
 *******************************************************************************
 * Project:		MQTTAct1
 *
 * Author:		Walter Trojan
 *
 * Description: Actor with four channels
 * 				Sending to MQTT
 *
 * IDE:			Eclipse
 *
 * Finished:	xx.xx.xx
 *
 * 11.09.16		user_main_71.c
 *				bernahme von MQTTSens1
 *				Blinking LED
 *
 * 12.09.16		user_main_72.c
 * 				Erweiterung von pwm.h und pmw.c von 3 auf 4 pwm-Kanle
 * 				Rchtecksignal auf allen 4 Kanlen (12,13,14,5)
 *
 * 14.09.16		user_main_73.c
 * 				Alle 4 Kanle mit 2A geprft
 * 				Mit auf/abschwellendem PWM Signal
 *
 * 14.09.16     user_main_74.c
 * 				Mit Anbindung an MQTT
 * 				Topic: actor1/ch0..3  also actor1/ch3
 * 				Data:  ch0..3 0..255  also ch3 255
 *
 *
 * Info:		On the ESP8266 the pins 0, 2, 4, 5, 12, 13, 14 and 15 all support PWM.
 * 				The limitation is that they must all be at the same frequency,
 * 				and the frequency must be between 1Hz and 1kHz.
 *
 *******************************************************************************/


#include <ets_sys.h>
#include <osapi.h>
#include <os_type.h>
#include <user_interface.h>
#include <espconn.h>
#include "mqtt.h"
#include "debug.h"
#include "mem.h"
#include "uart.h"
#include "uart_register.h"
#include "config.h"
#include "uart.h"
#include "driver/dht22.h"
#include "driver/ds18b20.h"
#include "driver/pwm.h"

#define LED_GN_MUX		PERIPHS_IO_MUX_MTDO_U
#define LED_GN_FUNC		FUNC_GPIO15
#define LED_GN_PIN		15

/*******************************************************************************
 * Global variables
 ******************************************************************************/
MQTT_Client mqttClient;
static ETSTimer Wloop;
static ETSTimer Systic;


bool     Gopub;
bool     GoDS;
uint32_t Systim = 0;
uint8_t  Pwmtim = 0;
bool     Pwmdir = true;
uint8_t  Pwmval = 0;
bool	 RdyInit;
bool     RdyCon;
bool	 RdyIP;
bool     RdyMQ;
bool	 RdyPub;
bool     RdySub = false;
char     TopBuf[20];
char     DatBuf[20];
uint32_t Benchcount;
char     msg[64];

char 	 temp[10];
char 	 hum[10];
struct   dht_sensor_data* r;
float    lastTemp, lastHum;
uint16_t Adcval;

uint8_t data[12];
uint8_t DScon[5];
uint8_t DSpin[5];
uint8_t DStemp [5];
char    DSsign [5];

const char *FlashSizeMap[] =
{
		"512 KB (256 KB + 256 KB)",	// 0x00
		"256 KB",			// 0x01
		"1024 KB (512 KB + 512 KB)", 	// 0x02
		"2048 KB (512 KB + 512 KB)"	// 0x03
		"4096 KB (512 KB + 512 KB)"	// 0x04
		"2048 KB (1024 KB + 1024 KB)"	// 0x05
		"4096 KB (1024 KB + 1024 KB)"	// 0x06
};

const char *WiFiMode[] =
{
		"NULL",		// 0x00
		"STATION",	// 0x01
		"SOFTAP", 	// 0x02
		"STATIONAP"	// 0x03
};

const char *WiFiStatus[] =
{
	    "STATION_IDLE", 			// 0x00
	    "STATION_CONNECTING", 		// 0x01
	    "STATION_WRONG_PASSWORD", 	// 0x02
	    "STATION_NO_AP_FOUND", 		// 0x03
	    "STATION_CONNECT_FAIL", 	// 0x04
	    "STATION_GOT_IP" 			// 0x05
};


uint8_t GetPwm(void)
{
	char valstr[4];
	uint8_t val;

	if((DatBuf[4] >= '0') && (DatBuf[4] <= '9'))
		valstr[0] = DatBuf[4];
	else{
		valstr[0] = '0';					// terminate valstr
		valstr[1] = 0;
		DatBuf[5] = 0;
	}
	if((DatBuf[5] >= '0') && (DatBuf[5] <= '9'))
		valstr[1] = DatBuf[5];
	else
		valstr[1] = 0;
	if((DatBuf[6] >= '0') && (DatBuf[6] <= '9')){
		valstr[2] = DatBuf[2];
		valstr[3] = 0;
	}
	else
		valstr[2] = 0;

	val = atoi(valstr);

	return(val);
}

/*******************************************************************************
 * WiFi_Connect: Connect station to router
 ******************************************************************************/
void ICACHE_FLASH_ATTR WIFI_Connect(uint8_t* ssid, uint8_t* pass)
{
	struct station_config stationConf;
	bool retval;								//

	INFO("WIFI_INIT\r\n");
	wifi_set_opmode(STATION_MODE);
	wifi_station_disconnect();					//
	wifi_station_dhcpc_stop();					//
	os_memset(&stationConf, 0, sizeof(struct station_config));

	os_sprintf(stationConf.ssid, "%s", ssid);
	//INFO("...SSID=%s\n\r",ssid);				//
	os_sprintf(stationConf.password, "%s", pass);
	//INFO("...PASS=%s\n\r",pass);
	stationConf.bssid_set = 0; 					//need not check MAC address of AP

	retval = wifi_station_set_config(&stationConf);
	if(retval)									//
		INFO("wifi_station_set = OK\n\r");

	retval = wifi_station_connect();
	wifi_station_dhcpc_start();					//
	if(retval)									//
		INFO("wifi_station_connect = OK\n\r");
	else
		INFO("wifi_station_connect = NNNOK\n\r");
}



/*******************************************************************************
 * MQTT callbacks
 ******************************************************************************/
void mqtt_connected_cb(uint32_t *args)
{

	os_delay_us(65000);
	INFO(" ==> MQTT: Connected\r\n");
	MQTT_Client* client = (MQTT_Client*)args;
	RdySub = false;
	RdyMQ  = true;

	strcpy(msg,"Actor1 started successfully");
	MQTT_Publish(&mqttClient, "Actor1/Status", msg, strlen(msg), 0, 0);

	MQTT_Subscribe(&mqttClient, "actor1/ch0", 0);
	MQTT_Subscribe(&mqttClient, "actor1/ch1", 0);
	MQTT_Subscribe(&mqttClient, "actor1/ch2", 0);
	MQTT_Subscribe(&mqttClient, "actor1/ch3", 0);


/*
	TimeSta = system_get_time();
	Benchcount = 1;
	os_sprintf(msg, "\r\nStart \r\n%4u  ESP01 Bench 0123456789012345678901234567890", Benchcount);
	MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);

*/
}

void mqtt_disconnected_cb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	RdyMQ = false;
	INFO(" ==> MQTT: Disconnected\r\n");
}

void mqtt_published_cb(uint32_t *args)
{
	MQTT_Client* client = (MQTT_Client*)args;
	INFO("MQTT: Published\r\n");
/*
	Benchcount = Benchcount + 1;
	if(Benchcount < 101){
		os_sprintf(msg, "%4u  ESP01 Bench 0123456789012345678901234567890", Benchcount);
		MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);
	}

	if(Benchcount == 101){
		TimeEnd = system_get_time();
		TimeGes = TimeEnd - TimeSta;
		os_sprintf(msg, "==> Zeit:%4u uS ", TimeGes);
		MQTT_Publish(&mqttClient, "ESP01/bench", msg, strlen(msg), 0, 0);
	}
*/
}

void mqtt_data_cb(uint32_t *args, const char* topic, uint32_t topic_len, const char *data, uint32_t data_len)
{
	char *topicBuf = (char*)os_zalloc(topic_len+1),
			*dataBuf = (char*)os_zalloc(data_len+1);

	MQTT_Client* client = (MQTT_Client*)args;

	os_memcpy(topicBuf, topic, topic_len);
	topicBuf[topic_len] = 0;

	os_memcpy(dataBuf, data, data_len);
	dataBuf[data_len] = 0;

	INFO("/r/n Receive topic: %s, data: %s \r\n", topicBuf, dataBuf);

	RdySub = true;
	strcpy(TopBuf,topicBuf);
	strcpy(DatBuf,dataBuf);

	os_free(topicBuf);
	os_free(dataBuf);
}


/*******************************************************************************
 * init_done_cb: Called after init is ready
 ******************************************************************************/

static void ICACHE_FLASH_ATTR init_done_cb(void)
{
	os_delay_us(65000);
	INFO("\r\n ==> Init done.\r\n");
	WIFI_Connect(config.sta_ssid, config.sta_pwd);
}


/*******************************************************************************
 * eventHandler: Called after some events
 ******************************************************************************/
static void ICACHE_FLASH_ATTR eventHandler(System_Event_t *event) {
	switch(event->event) {
		case EVENT_STAMODE_CONNECTED:
		{
			INFO(" ==> Event: EVENT_STAMODE_CONNECTED\r\n");
			break;
		}
		case EVENT_STAMODE_DISCONNECTED:
		{
			INFO(" ==> Event: EVENT_STAMODE_DISCONNECTED\r\n");
			break;
		}
		case EVENT_STAMODE_AUTHMODE_CHANGE:
		{
			INFO(" Event: EVENT_STAMODE_AUTHMODE_CHANGE\r\n");
			break;
		}
		case EVENT_STAMODE_GOT_IP:
		{
			INFO(" ==> Event: EVENT_STAMODE_GOT_IP\r\n");
			os_delay_us(65000);
			MQTT_Connect(&mqttClient);
			break;
		}
		case EVENT_SOFTAPMODE_STACONNECTED:
		{
			INFO(" Event: EVENT_SOFTAPMODE_STACONNECTED\r\n");
			break;
		}
		case EVENT_SOFTAPMODE_STADISCONNECTED:
		{
			INFO(" Event: EVENT_SOFTAPMODE_STADISCONNECTED\r\n");
			break;
		}
		default:
		{
			INFO(" Unexpected event: %d\r\n", event->event);
			break;
		}
	}
}





/********************************************************************************
 * sysclk: Will be called cyclic by RTOS, depending on timer Systic (every 10 ms)
 ******************************************************************************/

LOCAL void ICACHE_FLASH_ATTR sysclk(void *arg)
{
	Systim++;
	//Pwmtim++;
}



/*******************************************************************************
 * workloop: Will be by RTOS, depending on timer Wloop, calling itself, if ready
 ******************************************************************************/

static void ICACHE_FLASH_ATTR workloop(void *arg)
{
	os_timer_disarm(&Wloop);

	if((RdyMQ) && (RdySub)){
		RdySub = false;
		Pwmval = 0;
		INFO(" ==> Topic=%s\r\n",TopBuf);
		INFO(" ==> Data =%s\r\n",DatBuf);

		if(TopBuf[9] == '0'){							// Chan 0
			Pwmval = GetPwm();
			pwm_set_duty(Pwmval,0);
			pwm_start();
		}


		if(TopBuf[9] == '1'){							// Chan 1
			Pwmval = GetPwm();
			pwm_set_duty(Pwmval,1);
			pwm_start();
		}


		if(TopBuf[9] == '2'){							// Chan 2
			Pwmval = GetPwm();
			pwm_set_duty(Pwmval,2);
			pwm_start();
		}


		if(TopBuf[9] == '3'){							// Chan 3
			Pwmval = GetPwm();
			pwm_set_duty(Pwmval,3);
			pwm_start();
		}

	}

	if((Systim >= 0) && (Systim <= 5))						// blink green LED
		GPIO_OUTPUT_SET(LED_GN_PIN, 1);

	if((Systim >= 50) && (Systim <= 55))
		GPIO_OUTPUT_SET(LED_GN_PIN, 0);

	if(Systim >= 100)
		Systim = 0;

/*
											// PWM-Test
	pwm_set_duty(Pwmtim,0);					// chan 0
	pwm_set_duty(Pwmtim,1);					// chan 1
	pwm_set_duty(Pwmtim,2);					// chan 2
	pwm_set_duty(Pwmtim,3);					// chan 3
	pwm_start();

	if(Pwmtim == 255)
		Pwmdir = false;
	if(Pwmtim == 0)
		Pwmdir = true;
	if(Pwmdir)
		Pwmtim++;
	else
	 	Pwmtim--;
*/

	os_timer_setfn(&Wloop, (os_timer_func_t *)workloop, NULL);
	os_timer_arm(&Wloop, 10, 0);

}


/*******************************************************************************
 * user_init: Performs once for initialization
 ******************************************************************************/
void user_init(void)
{


	uart_init(BIT_RATE_115200, BIT_RATE_115200);
	os_delay_us(65000);

	system_set_os_print(1);
	INFO("\r\n\n\n==== System info: ====\n");
	INFO("SDK version:%s rom %d\n", system_get_sdk_version(), system_upgrade_userbin_check());
	INFO("Time = %ld\n", system_get_time());
	INFO("Chip id = 0x%x\n", system_get_chip_id());
	INFO("CPU freq = %d MHz\n", system_get_cpu_freq());
	INFO("Flash size map = %s\n", FlashSizeMap[system_get_flash_size_map()]);
	INFO("Free heap size = %d\n", system_get_free_heap_size());
	INFO("==== End System info ====\n\n");
	os_delay_us(6000);

	PIN_FUNC_SELECT(LED_GN_MUX,  LED_GN_FUNC);

	PIN_FUNC_SELECT(PWM_0_OUT_IO_MUX, PWM_0_OUT_IO_FUNC); // chan 0
	PIN_FUNC_SELECT(PWM_1_OUT_IO_MUX, PWM_1_OUT_IO_FUNC); // chan 1
	PIN_FUNC_SELECT(PWM_2_OUT_IO_MUX, PWM_2_OUT_IO_FUNC); // chan 2
	PIN_FUNC_SELECT(PWM_3_OUT_IO_MUX, PWM_3_OUT_IO_FUNC); // chan 3

	Systim = 0;
	Pwmtim = 0;
	Pwmdir = true;
	GPIO_OUTPUT_SET(LED_GN_PIN, 0);

	uint8_t duty[] = {0,0,0,0};
	pwm_init(500, duty); 					// 500 Hz, already calls pwm_start()
	pwm_set_duty(0,0);						// chan 0
	pwm_set_duty(0,1);						// chan 1
	pwm_set_duty(0,2);						// chan 2
	pwm_set_duty(0,3);						// chan 3
	pwm_start();

	wifi_station_set_auto_connect(0);				// no auto connection
	system_init_done_cb(init_done_cb);				// register init_done_cb
	wifi_set_event_handler_cb(eventHandler);		// register event handler

	config_load();

	RdyInit = false;
	RdyCon  = false;
	RdyIP   = false;
	RdyMQ   = false;
	RdyPub  = false;
	RdySub  = false;


	MQTT_InitConnection(&mqttClient, config.mqtt_host, config.mqtt_port, config.security);
	MQTT_InitClient(&mqttClient, config.device_id, config.mqtt_user, config.mqtt_pass, config.mqtt_keepalive, 1);
	MQTT_InitLWT(&mqttClient, "/lwt", "offline", 0, 0);
	MQTT_OnConnected(&mqttClient, mqtt_connected_cb);
	MQTT_OnDisconnected(&mqttClient, mqtt_disconnected_cb);
	MQTT_OnPublished(&mqttClient, mqtt_published_cb);
	MQTT_OnData(&mqttClient, mqtt_data_cb);



	os_timer_disarm(&Wloop);
	os_timer_setfn(&Wloop, (os_timer_func_t *)workloop, NULL);
	os_timer_arm(&Wloop, 10, 0);

	os_timer_disarm(&Systic);
	os_timer_setfn(&Systic, (os_timer_func_t *)sysclk, NULL);
	os_timer_arm(&Systic, 10, 1);

	INFO("\r\nSystem started ...\r\n");
}

//void user_rf_pre_init(void) {}
