/*
 * Adaptation of Paul Stoffregen's One wire library to the ESP8266 and
 *
 * Fixed Mikhail Grigorev <sleuthhound@gmail.com>
 * 
 * Paul's original library site:
 *   http://www.pjrc.com/teensy/td_libs_OneWire.html
 * 
 * See also http://playground.arduino.cc/Learning/OneWire
 * 
 */

#include "ets_sys.h"
#include "os_type.h"
#include "mem.h"
#include "osapi.h"
#include "user_interface.h"

#include "espconn.h"
#include "gpio.h"
#include "driver/ds18b20.h"

void ICACHE_FLASH_ATTR DS_Gpio()
{
	// Set DS18B20_PIN as gpio pin
	PIN_FUNC_SELECT(DSvorl_MUX,  DSvorl_FUNC);
	// Disable pull-down
//	PIN_PULLDWN_DIS(DSvorl_MUX);
	// Enable pull-up
	PIN_PULLUP_DIS(DSvorl_MUX);
	// Set DS18B20_PIN pin as an input
	GPIO_DIS_OUTPUT(DSvorl_PIN);

	PIN_FUNC_SELECT(DSback_MUX,  DSback_FUNC);
//	PIN_PULLDWN_DIS(DSback_MUX);
	PIN_PULLUP_DIS(DSback_MUX);
	GPIO_DIS_OUTPUT(DSback_PIN);

}

// Perform the one wire reset function.  We will wait up to 250uS for
// the bus to come high, if it doesn't then it is broken or shorted
// and we return a 0;
// Returns 1 if a device asserted a presence pulse, 0 otherwise.
uint8_t ICACHE_FLASH_ATTR DS_reset(uint8_t gpio_pin)
{
	int r;
	uint8_t retries = 125;
	GPIO_DIS_OUTPUT(gpio_pin);
	do {
		if (--retries == 0) return 0;
		os_delay_us(2);
	} while ( !GPIO_INPUT_GET(gpio_pin));
	GPIO_OUTPUT_SET(gpio_pin, 0);
	os_delay_us(500);
	GPIO_DIS_OUTPUT(gpio_pin);
	os_delay_us(65);
	r = !GPIO_INPUT_GET(gpio_pin);
	os_delay_us(490);
	return r;
}

//
// Write a byte. The writing code uses the active drivers to raise the
// pin high, if you need power after the write (e.g. DS18S20 in
// parasite power mode) then set 'power' to 1, otherwise the pin will
// go tri-state at the end of the write to avoid heating in a short or
// other mishap.
//
void ICACHE_FLASH_ATTR DS_write(uint8_t v, int power, uint8_t gpio_pin)
{
	uint8_t bitMask;
	for (bitMask = 0x01; bitMask; bitMask <<= 1) {
		DS_write_bit((bitMask & v)?1:0, gpio_pin);
	}

	if (!power) {
		GPIO_DIS_OUTPUT(gpio_pin);
		GPIO_OUTPUT_SET(gpio_pin, 0);
	}
}

//
// Write a bit. Port and bit is used to cut lookup time and provide
// more certain timing.
//
void ICACHE_FLASH_ATTR DS_write_bit(int v, uint8_t gpio_pin)
{
	GPIO_OUTPUT_SET(gpio_pin, 0);
	if(v) {
		os_delay_us(10);
		GPIO_OUTPUT_SET(gpio_pin, 1);
		os_delay_us(55);
	} else {
		os_delay_us(65);
		GPIO_OUTPUT_SET(gpio_pin, 1);
		os_delay_us(5);
	}
}

//
// Read a byte
//
uint8_t ICACHE_FLASH_ATTR DS_read(uint8_t gpio_pin)
{
	uint8_t bitMask;
	uint8_t r = 0;
	for (bitMask = 0x01; bitMask; bitMask <<= 1) {
		if ( DS_read_bit(gpio_pin)) r |= bitMask;
	}
	return r;
}

//
// Read a bit. Port and bit is used to cut lookup time and provide
// more certain timing.
//
int ICACHE_FLASH_ATTR DS_read_bit(uint8_t gpio_pin)
{
	int r;
	GPIO_OUTPUT_SET(gpio_pin, 0);
	os_delay_us(3);
	GPIO_DIS_OUTPUT(gpio_pin);
	os_delay_us(10);
	r = GPIO_INPUT_GET(gpio_pin);
	os_delay_us(53);
	return r;
}
